@@ -16,4 +16,8 @@ mod ping_result;
 mod ping_worker;
 mod rnp_core;
 mod rnp_core_config;
-mod rnp_utils;
\ No newline at end of file
+mod rnp_utils;
+mod rnp_dto;
+
+#[cfg(test)]
+mod rnp_test_common;
