@@ -2,28 +2,13 @@ use std::time::Duration;
 use std::net::SocketAddr;
 use async_trait::async_trait;
 
-#[derive(Debug)]
-pub struct PingClientPingResultDetails {
-    pub actual_local_addr: Option<SocketAddr>,
-    pub round_trip_time: Duration,
-    pub is_timeout: bool,
-    pub handshake_error: Option<Box<dyn std::error::Error + Send>>,
-}
+#[derive(thiserror::Error, Debug)]
+pub enum PingClientWarning {
+    #[error("{0}")]
+    DisconnectFailed(Box<dyn std::error::Error + Send>),
 
-impl PingClientPingResultDetails {
-    pub fn new(
-        actual_local_addr: Option<SocketAddr>,
-        round_trip_time: Duration,
-        is_timeout: bool,
-        handshake_error: Option<Box<dyn std::error::Error + Send>>,
-    ) -> PingClientPingResultDetails {
-        PingClientPingResultDetails {
-            actual_local_addr,
-            round_trip_time,
-            is_timeout,
-            handshake_error,
-        }
-    }
+    #[error("{0}")]
+    AppHandshakeFailed(Box<dyn std::error::Error + Send>),
 }
 
 #[derive(thiserror::Error, Debug)]
@@ -35,6 +20,31 @@ pub enum PingClientError {
     PingFailed(Box<dyn std::error::Error + Send>),
 }
 
+
+#[derive(Debug)]
+pub struct PingClientPingResultDetails {
+    pub actual_local_addr: Option<SocketAddr>,
+    pub round_trip_time: Duration,
+    pub is_timeout: bool,
+    pub warning: Option<PingClientWarning>,
+}
+
+impl PingClientPingResultDetails {
+    pub fn new(
+        actual_local_addr: Option<SocketAddr>,
+        round_trip_time: Duration,
+        is_timeout: bool,
+        warning: Option<PingClientWarning>,
+    ) -> PingClientPingResultDetails {
+        PingClientPingResultDetails {
+            actual_local_addr,
+            round_trip_time,
+            is_timeout,
+            warning,
+        }
+    }
+}
+
 pub type PingClientResult<T, E = PingClientError> = std::result::Result<T, E>;
 
 #[async_trait]
