@@ -113,8 +113,8 @@ impl PingWorker {
             !ping_result.is_timeout,
             ping_result.round_trip_time,
             ping_result.is_timeout,
+            ping_result.warning,
             None,
-            ping_result.handshake_error,
         );
 
         self.result_sender.send(result).await.unwrap();
@@ -139,8 +139,8 @@ impl PingWorker {
             false,
             Duration::from_millis(0),
             false,
-            Some(error),
             None,
+            Some(error),
         );
 
         self.result_sender.send(result).await.unwrap();
